@@ -1,10 +1,9 @@
 # SPDX-License-Identifier: BSD-3-Clause
 # Copyright (c) 2023 Scipp contributors (https://github.com/scipp)
 # @author Simon Heybrock
-"""Sub-package for lower-level control over binning and histogramming.
-"""
+"""Sub-package for lower-level control over binning and histogramming."""
 
-# flake8: noqa
+# ruff: noqa: F403
 from ..core.binning import make_binned, make_histogrammed
 
 # This makes Sphinx display these functions in the docs.
