@@ -3,26 +3,15 @@
 # @author Jan-Lukas Wynen
 
 import dataclasses
-from enum import Enum, auto
-from typing import Optional
 
 from ..core import Variable
 
 
-class Destination(Enum):
-    coord = auto()
-    attr = auto()
-
-    @property
-    def other(self):
-        return Destination.attr if self == Destination.coord else Destination.coord
-
-
 @dataclasses.dataclass
 class Coord:
-    dense: Optional[Variable]  # for dense variable or bin-coord
-    event: Optional[Variable]
-    destination: Destination
+    dense: Variable | None  # for dense variable or bin-coord
+    event: Variable | None
+    aligned: bool
     usages: int = -1  # negative for unlimited usages
 
     @property
@@ -33,7 +22,7 @@ class Coord:
     def has_event(self) -> bool:
         return self.event is not None
 
-    def use(self):
+    def use(self) -> None:
         if self.usages > 0:
             self.usages -= 1
 
@@ -42,4 +31,8 @@ class Coord:
         return self.usages == 0
 
     def has_dim(self, dim: str) -> bool:
-        return dim in (self.dense.dims if self.has_dense else self.event.dims)
+        if self.dense is not None:
+            return dim in self.dense.dims
+        if self.event is not None:
+            return dim in self.event.dims
+        return False
