@@ -2,17 +2,20 @@
 # Copyright (c) 2023 Scipp contributors (https://github.com/scipp)
 # @author Simon Heybrock
 
-from typing import Literal, Optional
+from typing import Literal
 
 from .._scipp import core as _cpp
 from ..typing import VariableLikeType
 from ._cpp_wrapper_util import call_func as _call_cpp_func
 from .concepts import transform_data
+from .cpp_classes import Variable
 
 
-def cumsum(a: VariableLikeType,
-           dim: Optional[str] = None,
-           mode: Literal['exclusive', 'inclusive'] = 'inclusive') -> VariableLikeType:
+def cumsum(
+    a: VariableLikeType,
+    dim: str | None = None,
+    mode: Literal['exclusive', 'inclusive'] = 'inclusive',
+) -> VariableLikeType:
     """Return the cumulative sum along the specified dimension.
 
     See :py:func:`scipp.sum` on how rounding errors for float32 inputs are handled.
@@ -34,10 +37,10 @@ def cumsum(a: VariableLikeType,
         The cumulative sum of the input values.
     """
 
-    def _cumsum(data):
+    def _cumsum(data: Variable) -> Variable:
         if dim is None:
-            return _call_cpp_func(_cpp.cumsum, data, mode=mode)
+            return _call_cpp_func(_cpp.cumsum, data, mode=mode)  # type: ignore[return-value]
         else:
-            return _call_cpp_func(_cpp.cumsum, data, dim, mode=mode)
+            return _call_cpp_func(_cpp.cumsum, data, dim, mode=mode)  # type: ignore[return-value]
 
     return transform_data(a, _cumsum)
