@@ -3,20 +3,24 @@
 # @author Matthew Andrew
 from __future__ import annotations
 
-from typing import Any, Literal, Optional, TypeVar, Union, overload
+from typing import Any, Literal, TypeVar, overload
 
 from .._scipp import core as _cpp
-from ..typing import VariableLikeType
+from ..typing import ScippIndex, VariableLikeType
 from ._cpp_wrapper_util import call_func as _call_cpp_func
 from .comparison import identical
-from .cpp_classes import Dataset, DatasetError, Variable
+from .cpp_classes import DataArray, Dataset, DatasetError, Variable
 from .data_group import DataGroup
 from .unary import to_unit
 
-_ContainerWithCoords = TypeVar('_ContainerWithCoords', _cpp.DataArray, _cpp.Dataset)
+_T = TypeVar('_T', Variable, DataGroup[object])
+_VarDa = TypeVar('_VarDa', Variable, DataArray)
+_DsDg = TypeVar('_DsDg', Dataset, DataGroup[object])
+_E1 = TypeVar('_E1')
+_E2 = TypeVar('_E2')
 
 
-def islinspace(x: Variable, dim: str = None) -> Variable:
+def islinspace(x: _T, dim: str | None = None) -> _T:
     """Check if the values of a variable are evenly spaced.
 
     Parameters
@@ -33,14 +37,14 @@ def islinspace(x: Variable, dim: str = None) -> Variable:
         spaced values, variable of value False otherwise.
     """
     if dim is None:
-        return _call_cpp_func(_cpp.islinspace, x)
+        return _call_cpp_func(_cpp.islinspace, x)  # type: ignore[return-value]
     else:
-        return _call_cpp_func(_cpp.islinspace, x, dim)
+        return _call_cpp_func(_cpp.islinspace, x, dim)  # type: ignore[return-value]
 
 
-def issorted(x: Variable,
-             dim: str,
-             order: Literal['ascending', 'descending'] = 'ascending') -> Variable:
+def issorted(
+    x: _T, dim: str, order: Literal['ascending', 'descending'] = 'ascending'
+) -> _T:
     """Check if the values of a variable are sorted.
 
     - If ``order`` is 'ascending',
@@ -68,12 +72,28 @@ def issorted(x: Variable,
     --------
     scipp.allsorted
     """
-    return _call_cpp_func(_cpp.issorted, x, dim, order)
+    return _call_cpp_func(_cpp.issorted, x, dim, order)  # type: ignore[return-value]
 
 
-def allsorted(x: Variable,
-              dim: str,
-              order: Literal['ascending', 'descending'] = 'ascending') -> bool:
+@overload
+def allsorted(
+    x: Variable, dim: str, order: Literal['ascending', 'descending'] = 'ascending'
+) -> bool: ...
+
+
+@overload
+def allsorted(
+    x: DataGroup[object],
+    dim: str,
+    order: Literal['ascending', 'descending'] = 'ascending',
+) -> DataGroup[object]: ...
+
+
+def allsorted(
+    x: Variable | DataGroup[object],
+    dim: str,
+    order: Literal['ascending', 'descending'] = 'ascending',
+) -> bool | DataGroup[object]:
     """Check if all values of a variable are sorted.
 
     - If ``order`` is 'ascending',
@@ -103,9 +123,11 @@ def allsorted(x: Variable,
     return _call_cpp_func(_cpp.allsorted, x, dim, order)
 
 
-def sort(x: VariableLikeType,
-         key: Union[str, Variable],
-         order: Literal['ascending', 'descending'] = 'ascending') -> VariableLikeType:
+def sort(
+    x: VariableLikeType,
+    key: str | Variable,
+    order: Literal['ascending', 'descending'] = 'ascending',
+) -> VariableLikeType:
     """Sort variable along a dimension by a sort key or dimension label.
 
     - If ``order`` is 'ascending',
@@ -132,7 +154,7 @@ def sort(x: VariableLikeType,
     scipp.DimensionError
         If the key is a Variable that does not have exactly 1 dimension.
     """
-    return _call_cpp_func(_cpp.sort, x, key, order)
+    return _call_cpp_func(_cpp.sort, x, key, order)  # type: ignore[return-value]
 
 
 def values(x: VariableLikeType) -> VariableLikeType:
@@ -152,7 +174,7 @@ def values(x: VariableLikeType) -> VariableLikeType:
     --------
     scipp.variances, scipp.stddevs
     """
-    return _call_cpp_func(_cpp.values, x)
+    return _call_cpp_func(_cpp.values, x)  # type: ignore[return-value]
 
 
 def variances(x: VariableLikeType) -> VariableLikeType:
@@ -173,7 +195,7 @@ def variances(x: VariableLikeType) -> VariableLikeType:
     --------
     scipp.values, scipp.stddevs
     """
-    return _call_cpp_func(_cpp.variances, x)
+    return _call_cpp_func(_cpp.variances, x)  # type: ignore[return-value]
 
 
 def stddevs(x: VariableLikeType) -> VariableLikeType:
@@ -194,36 +216,55 @@ def stddevs(x: VariableLikeType) -> VariableLikeType:
     --------
     scipp.values, scipp.variances
     """
-    return _call_cpp_func(_cpp.stddevs, x)
+    return _call_cpp_func(_cpp.stddevs, x)  # type: ignore[return-value]
 
 
-def where(condition: Variable, x: Variable, y: Variable) -> Variable:
+@overload
+def where(
+    condition: Variable | DataArray, x: DataArray, y: Variable | DataArray
+) -> DataArray: ...
+
+
+@overload
+def where(
+    condition: Variable | DataArray, x: Variable | DataArray, y: DataArray
+) -> DataArray: ...
+
+
+@overload
+def where(condition: Variable | DataArray, x: Variable, y: Variable) -> Variable: ...
+
+
+def where(
+    condition: Variable | DataArray, x: Variable | DataArray, y: Variable | DataArray
+) -> Variable | DataArray:
     """Return elements chosen from x or y depending on condition.
 
     Parameters
     ----------
     condition:
-        Variable with dtype=bool.
+        Array with dtype=bool.
     x:
-        Variable with values from which to choose.
+        Array with values from which to choose.
     y:
-        Variable with values from which to choose.
+        Array with values from which to choose.
 
     Returns
     -------
     :
-        Variable with elements from x where condition is True
+        Array with elements from x where condition is True
         and elements from y elsewhere.
     """
-    return _call_cpp_func(_cpp.where, condition, x, y)
+    return _call_cpp_func(_cpp.where, condition, x, y)  # type: ignore[return-value]
 
 
 def to(
-        var: VariableLikeType,  # noqa
-        *,
-        unit: Optional[Union[_cpp.Unit, str]] = None,
-        dtype: Optional[Any] = None,
-        copy: bool = True) -> VariableLikeType:
+    var: Variable,
+    *,
+    unit: _cpp.Unit | str | None = None,
+    dtype: Any | None = None,
+    copy: bool = True,
+) -> Variable:
     """Converts a Variable or DataArray to a different dtype and/or a different unit.
 
     If the dtype and unit are both unchanged and ``copy`` is `False`,
@@ -296,31 +337,21 @@ def to(
         return to_unit(var, unit=unit, copy=copy).astype(dtype, copy=False)
 
 
-@overload
-def merge(lhs: Dataset, rhs: Dataset) -> Dataset:
-    ...
-
-
-@overload
-def merge(lhs: DataGroup, rhs: DataGroup) -> DataGroup:
-    ...
-
-
-def merge(lhs, rhs):
+def merge(lhs: _DsDg, rhs: _DsDg) -> _DsDg:
     """Merge two datasets or data groups into one.
 
     If an item appears in both inputs, it must have an identical value in both.
 
     Parameters
     ----------
-    lhs:
+    lhs: Dataset | DataGroup
         First dataset or data group.
-    rhs:
+    rhs: Dataset | DataGroup
         Second dataset or data group.
 
     Returns
     -------
-    :
+    : Dataset | DataGroup
         A new object that contains the union of all data items,
         coords, masks and attributes.
 
@@ -329,8 +360,9 @@ def merge(lhs, rhs):
     scipp.DatasetError
         If there are conflicting items with different content.
     """
-    if isinstance(lhs, Dataset) or isinstance(rhs, Dataset):
-        return _call_cpp_func(_cpp.merge, lhs, rhs)
+    # Check both arguments to make `_cpp.merge` raise TypeError on mismatch.
+    if isinstance(lhs, Dataset) or isinstance(rhs, Dataset):  # type: ignore[redundant-expr]
+        return _call_cpp_func(_cpp.merge, lhs, rhs)  # type: ignore[return-value]
     return _merge_data_group(lhs, rhs)
 
 
@@ -339,16 +371,56 @@ def _generic_identical(a: Any, b: Any) -> bool:
         return identical(a, b)
     except TypeError:
         from numpy import array_equal
+
         try:
             return array_equal(a, b)
         except TypeError:
-            return a == b
+            return a == b  # type: ignore[no-any-return]
 
 
-def _merge_data_group(lhs: DataGroup, rhs: DataGroup) -> DataGroup:
-    res = DataGroup(dict(lhs))
+def _merge_data_group(lhs: DataGroup[_E1], rhs: DataGroup[_E2]) -> DataGroup[_E1 | _E2]:
+    res: DataGroup[_E1 | _E2] = DataGroup(dict(lhs))
     for k, v in rhs.items():
         if k in res and not _generic_identical(res[k], v):
             raise DatasetError(f"Cannot merge data groups. Mismatch in item {k}")
         res[k] = v
     return res
+
+
+def _raw_positional_index(
+    sizes: dict[str, int], coord: Variable, index: slice | Variable
+) -> tuple[str, list[int | Variable]]:
+    dim, *inds = _call_cpp_func(
+        _cpp.label_based_index_to_positional_index,
+        list(sizes.keys()),
+        list(sizes.values()),
+        coord,
+        index,
+    )
+    return dim, inds  # type: ignore[return-value]
+
+
+def label_based_index_to_positional_index(
+    sizes: dict[str, int],
+    coord: Variable,
+    index: slice | Variable,
+) -> ScippIndex:
+    """Returns the positional index equivalent to label based indexing
+    the coord with values."""
+    dim, inds = _raw_positional_index(sizes, coord, index)
+    # Length of inds is 1 if index was a variable
+    if len(inds) == 1:
+        if inds[0] < 0 or sizes[dim] <= inds[0]:
+            # If index is a variable and the coord is a bin-edge coord
+            #   - if the index is less than any edge then inds[0] is -1
+            #   - if the index is greater than any edge then inds[0] is sizes[dim]
+            raise IndexError(
+                f"Value {index} is not contained in the bin-edge coord {coord}"
+            )
+        return (dim, inds[0])
+    return (dim, slice(*inds))
+
+
+def as_const(x: _VarDa) -> _VarDa:
+    """Return a copy with the readonly flag set."""
+    return _call_cpp_func(_cpp.as_const, x)  # type: ignore[return-value]
