@@ -1,9 +1,10 @@
 # SPDX-License-Identifier: BSD-3-Clause
 # Copyright (c) 2023 Scipp contributors (https://github.com/scipp)
 # @author Matthew Andrew
-# flake8: noqa: E501
+# ruff: noqa: E501
 
-from typing import Dict, List, Optional, Sequence, Tuple, Union
+from collections.abc import Sequence
+from typing import overload
 
 import numpy as np
 
@@ -13,13 +14,32 @@ from . import data_group
 from ._cpp_wrapper_util import call_func as _call_cpp_func
 from ._sizes import _parse_dims_shape_sizes
 from .concepts import transform_data
+from .cpp_classes import Variable
+
+
+@overload
+def broadcast(
+    x: VariableLikeType,
+    *,
+    dims: Sequence[str],
+    shape: Sequence[int],
+) -> VariableLikeType: ...
+
+
+@overload
+def broadcast(
+    x: VariableLikeType,
+    *,
+    sizes: dict[str, int],
+) -> VariableLikeType: ...
 
 
 def broadcast(
     x: VariableLikeType,
-    dims: Optional[Union[List[str], Tuple[str, ...]]] = None,
-    shape: Optional[Sequence[int]] = None,
-    sizes: Optional[Dict[str, int]] = None,
+    *,
+    dims: Sequence[str] | None = None,
+    shape: Sequence[int] | None = None,
+    sizes: dict[str, int] | None = None,
 ) -> VariableLikeType:
     """Broadcast a Variable or a DataArray.
 
@@ -50,10 +70,12 @@ def broadcast(
     : Same type as input
         New Variable or DataArray with requested dimension labels and shape.
     """
-    sizes = _parse_dims_shape_sizes(dims=dims, shape=shape, sizes=sizes)
+    dims_and_shape = _parse_dims_shape_sizes(dims=dims, shape=shape, sizes=sizes)
+    dims = dims_and_shape["dims"]
+    shape = dims_and_shape["shape"]
 
-    def _broadcast(x):
-        return _call_cpp_func(_cpp.broadcast, x, sizes["dims"], sizes["shape"])
+    def _broadcast(data: Variable) -> Variable:
+        return _call_cpp_func(_cpp.broadcast, data, dims, shape)  # type: ignore[return-value]
 
     return transform_data(x, _broadcast)
 
@@ -110,22 +132,48 @@ def concat(x: Sequence[VariableLikeType], dim: str) -> VariableLikeType:
       <scipp.DataArray>
       Dimensions: Sizes[x:6, ]
       Coordinates:
-        x                           int64  [dimensionless]  (x)  [0, 1, ..., 100, 200]
+      * x                           int64  [dimensionless]  (x)  [0, 1, ..., 100, 200]
       Data:
                                     int64  [dimensionless]  (x)  [0, 1, ..., 100, 200]
       >>> z.values
       array([  0,   1,   2,   0, 100, 200])
     """
     if x and isinstance(x[0], data_group.DataGroup):
-        return data_group._apply_to_items(concat, x, dim)
-    return _call_cpp_func(_cpp.concat, x, dim)
+        return data_group.apply_to_items(
+            concat,
+            x,
+            dim,
+        )
+    return _call_cpp_func(_cpp.concat, x, dim)  # type: ignore[return-value]
 
 
-def fold(x: VariableLikeType,
-         dim: str,
-         sizes: Optional[Dict[str, int]] = None,
-         dims: Optional[Union[List[str], Tuple[str, ...]]] = None,
-         shape: Optional[Sequence[int]] = None) -> VariableLikeType:
+@overload
+def fold(
+    x: VariableLikeType,
+    dim: str,
+    *,
+    dims: Sequence[str],
+    shape: Sequence[int],
+) -> VariableLikeType: ...
+
+
+@overload
+def fold(
+    x: VariableLikeType,
+    dim: str,
+    *,
+    sizes: dict[str, int],
+) -> VariableLikeType: ...
+
+
+def fold(
+    x: VariableLikeType,
+    dim: str,
+    *,
+    dims: Sequence[str] | None = None,
+    shape: Sequence[int] | None = None,
+    sizes: dict[str, int] | None = None,
+) -> VariableLikeType:
     """Fold a single dimension of a variable or data array into multiple dims.
 
     One and only one of these sets of arguments must be given:
@@ -180,7 +228,7 @@ def fold(x: VariableLikeType,
       <scipp.DataArray>
       Dimensions: Sizes[y:2, z:3, ]
       Coordinates:
-        x                           int64  [dimensionless]  (y, z)  [0, 1, ..., 4, 5]
+      * x                           int64  [dimensionless]  (y, z)  [0, 1, ..., 4, 5]
       Data:
                                   float64  [dimensionless]  (y, z)  [0, 0.1, ..., 0.4, 0.5]
       >>> sc.fold(a, dim='x', sizes={'y': 2, 'z': 3}).data.values
@@ -190,41 +238,50 @@ def fold(x: VariableLikeType,
       array([[0, 1, 2],
              [3, 4, 5]])
     """
-    sizes = _parse_dims_shape_sizes(dims=dims, shape=shape, sizes=sizes)
+    dims_and_shape = _parse_dims_shape_sizes(dims=dims, shape=shape, sizes=sizes)
+    dims = dims_and_shape["dims"]
+    new_shape = list(dims_and_shape["shape"])
 
     # Handle potential size of -1.
     # Note that we implement this here on the Python layer, because one cannot create
     # a C++ Dimensions object with negative sizes.
-    new_shape = sizes["shape"]
     minus_one_count = new_shape.count(-1)
     if minus_one_count > 1:
         raise _cpp.DimensionError(
-            "Can only have a single -1 in the new requested shape.")
+            "Can only have a single -1 in the new requested shape."
+        )
     if minus_one_count == 1:
+        if (size := x.sizes[dim]) is None:
+            raise ValueError(
+                f"Dim {dim} has inconsistent size, cannot compute final shape."
+            )
         ind = new_shape.index(-1)
         new_shape[ind] = 1
         new_volume = np.prod(new_shape)
-        dim_size = x.sizes[dim] // new_volume
-        if x.sizes[dim] % new_volume != 0:
-            raise ValueError("-1 in new shape was computed to be {}, but the original "
-                             "shape {} cannot be divided by {}.".format(
-                                 dim_size, x.sizes[dim], dim_size))
+        dim_size = int(size // new_volume)
+        if size % new_volume != 0:
+            raise ValueError(
+                f"-1 in new shape was computed to be {dim_size}, but the original "
+                f"shape {size} cannot be divided by {dim_size}."
+            )
         new_shape[ind] = dim_size
 
-    return _call_cpp_func(_cpp.fold, x, dim, sizes["dims"], new_shape)
+    return _call_cpp_func(_cpp.fold, x, dim, dims, new_shape)  # type: ignore[return-value]
 
 
-def flatten(x: VariableLikeType,
-            dims: Optional[Union[List[str], Tuple[str, ...]]] = None,
-            to: Optional[str] = None) -> VariableLikeType:
+def flatten(
+    x: VariableLikeType,
+    dims: Sequence[str] | None = None,
+    to: str | None = None,
+) -> VariableLikeType:
     """Flatten multiple dimensions into a single dimension.
 
     If the input has a bin-edge coordinate that cannot be joined together it will not
     be included in the output.
 
-    If the input is a DataArray then coords, masks, and attrs that contain at least one
+    If the input is a DataArray then coords and masks that contain at least one
     of the flattened dimensions will also be flattened. This implies that when
-    flattening all dims, i.e., when ``dims=None``, all coords, masks, and attrs that
+    flattening all dims, i.e., when ``dims=None``, all coords and masks that
     share *some or all* dimensions with the data will be flattened.
 
     Parameters
@@ -281,18 +338,18 @@ def flatten(x: VariableLikeType,
       <scipp.DataArray>
       Dimensions: Sizes[x:2, y:3, ]
       Coordinates:
-        x                           int64  [dimensionless]  (x)  [0, 1]
-        xy                          int64  [dimensionless]  (x, y)  [0, 1, ..., 4, 5]
-        y                           int64  [dimensionless]  (y)  [0, 1, 2]
+      * x                           int64  [dimensionless]  (x)  [0, 1]
+      * xy                          int64  [dimensionless]  (x, y)  [0, 1, ..., 4, 5]
+      * y                           int64  [dimensionless]  (y)  [0, 1, 2]
       Data:
                                   float64  [dimensionless]  (x, y)  [0, 0.1, ..., 0.4, 0.5]
       >>> sc.flatten(a, to='u')
       <scipp.DataArray>
       Dimensions: Sizes[u:6, ]
       Coordinates:
-        x                           int64  [dimensionless]  (u)  [0, 0, ..., 1, 1]
-        xy                          int64  [dimensionless]  (u)  [0, 1, ..., 4, 5]
-        y                           int64  [dimensionless]  (u)  [0, 1, ..., 1, 2]
+      * x                           int64  [dimensionless]  (u)  [0, 0, ..., 1, 1]
+      * xy                          int64  [dimensionless]  (u)  [0, 1, ..., 4, 5]
+      * y                           int64  [dimensionless]  (u)  [0, 1, ..., 1, 2]
       Data:
                                   float64  [dimensionless]  (u)  [0, 0.1, ..., 0.4, 0.5]
 
@@ -303,12 +360,12 @@ def flatten(x: VariableLikeType,
         # makes more sense for the dims that we want to flatten to come first
         # in the argument list.
         raise ValueError("The final flattened dimension is required.")
-    return _call_cpp_func(_cpp.flatten, x, dims, to)
+    return _call_cpp_func(_cpp.flatten, x, dims, to)  # type: ignore[return-value]
 
 
 def transpose(
-        x: VariableLikeType,
-        dims: Optional[Union[List[str], Tuple[str, ...]]] = None) -> VariableLikeType:
+    x: VariableLikeType, dims: Sequence[str] | None = None
+) -> VariableLikeType:
     """Transpose dimensions of the input.
 
     Parameters
@@ -329,13 +386,12 @@ def transpose(
     scipp.DimensionError
         If ``dims`` are incompatible with the input data.
     """
-    return _call_cpp_func(_cpp.transpose, x, dims if dims is not None else [])
+    return _call_cpp_func(_cpp.transpose, x, dims if dims is not None else [])  # type: ignore[return-value]
 
 
 def squeeze(
-        x: VariableLikeType,
-        dim: Optional[Union[str, List[str], Tuple[str,
-                                                  ...]]] = None) -> VariableLikeType:
+    x: VariableLikeType, dim: str | Sequence[str] | None = None
+) -> VariableLikeType:
     """Remove dimensions of length 1.
 
     This is equivalent to indexing the squeezed dimensions with index 0, that is
@@ -367,7 +423,7 @@ def squeeze(
     Examples
     --------
 
-      >>> v = sc.arange('a', 3).fold('a', {'x': 1, 'y': 3, 'z': 1})
+      >>> v = sc.arange('a', 3).fold('a', sizes={'x': 1, 'y': 3, 'z': 1})
       >>> v
       <scipp.Variable> (x: 1, y: 3, z: 1)      int64  [dimensionless]  [0, 1, 2]
       >>> sc.squeeze(v)
@@ -377,7 +433,7 @@ def squeeze(
       >>> sc.squeeze(v, ['x', 'z'])
       <scipp.Variable> (y: 3)      int64  [dimensionless]  [0, 1, 2]
 
-    Coordinates for squeezed dimensions are turned into attributes:
+    Coordinates for squeezed dimensions become unaligned:
 
       >>> da = sc.DataArray(v, coords={'x': sc.arange('x', 1),
       ...                              'y': sc.arange('y', 3)})
@@ -385,18 +441,17 @@ def squeeze(
       <scipp.DataArray>
       Dimensions: Sizes[x:1, y:3, z:1, ]
       Coordinates:
-        x                           int64  [dimensionless]  (x)  [0]
-        y                           int64  [dimensionless]  (y)  [0, 1, 2]
+      * x                           int64  [dimensionless]  (x)  [0]
+      * y                           int64  [dimensionless]  (y)  [0, 1, 2]
       Data:
                                     int64  [dimensionless]  (x, y, z)  [0, 1, 2]
       >>> sc.squeeze(da)
       <scipp.DataArray>
       Dimensions: Sizes[y:3, ]
       Coordinates:
-        y                           int64  [dimensionless]  (y)  [0, 1, 2]
+        x                           int64  [dimensionless]  ()  0
+      * y                           int64  [dimensionless]  (y)  [0, 1, 2]
       Data:
                                     int64  [dimensionless]  (y)  [0, 1, 2]
-      Attributes:
-        x                           int64  [dimensionless]  ()  [0]
     """
-    return _call_cpp_func(_cpp.squeeze, x, (dim, ) if isinstance(dim, str) else dim)
+    return _call_cpp_func(_cpp.squeeze, x, (dim,) if isinstance(dim, str) else dim)  # type: ignore[return-value]
