@@ -4,16 +4,15 @@
 
 from __future__ import annotations
 
-from typing import Optional
-
 from .._scipp import core as _cpp
 from ..typing import VariableLikeType
 from ._cpp_wrapper_util import call_func as _call_cpp_func
+from .cpp_classes import Variable
 
 
-def sin(x: VariableLikeType,
-        *,
-        out: Optional[VariableLikeType] = None) -> VariableLikeType:
+def sin(
+    x: VariableLikeType, *, out: VariableLikeType | None = None
+) -> VariableLikeType:
     """Element-wise sine.
 
     The input must have a plane-angle unit, i.e. ``rad``, ``deg``.
@@ -30,12 +29,12 @@ def sin(x: VariableLikeType,
     : Same type as input
         The sine values of the input.
     """
-    return _call_cpp_func(_cpp.sin, x, out=out)
+    return _call_cpp_func(_cpp.sin, x, out=out)  # type: ignore[return-value]
 
 
-def cos(x: VariableLikeType,
-        *,
-        out: Optional[VariableLikeType] = None) -> VariableLikeType:
+def cos(
+    x: VariableLikeType, *, out: VariableLikeType | None = None
+) -> VariableLikeType:
     """Element-wise cosine.
 
     The input must have a plane-angle unit, i.e. ``rad``, ``deg``.
@@ -52,12 +51,12 @@ def cos(x: VariableLikeType,
     : Same type as input
         The cosine values of the input.
     """
-    return _call_cpp_func(_cpp.cos, x, out=out)
+    return _call_cpp_func(_cpp.cos, x, out=out)  # type: ignore[return-value]
 
 
-def tan(x: VariableLikeType,
-        *,
-        out: Optional[VariableLikeType] = None) -> VariableLikeType:
+def tan(
+    x: VariableLikeType, *, out: VariableLikeType | None = None
+) -> VariableLikeType:
     """Element-wise tangent.
 
     The input must have a plane-angle unit, i.e. ``rad``, ``deg``.
@@ -74,12 +73,12 @@ def tan(x: VariableLikeType,
     : Same type as input
         The tangent values of the input.
     """
-    return _call_cpp_func(_cpp.tan, x, out=out)
+    return _call_cpp_func(_cpp.tan, x, out=out)  # type: ignore[return-value]
 
 
-def asin(x: VariableLikeType,
-         *,
-         out: Optional[VariableLikeType] = None) -> VariableLikeType:
+def asin(
+    x: VariableLikeType, *, out: VariableLikeType | None = None
+) -> VariableLikeType:
     """Element-wise inverse sine.
 
     Parameters
@@ -94,12 +93,12 @@ def asin(x: VariableLikeType,
     : Same type as input
         The inverse sine values of the input in radians.
     """
-    return _call_cpp_func(_cpp.asin, x, out=out)
+    return _call_cpp_func(_cpp.asin, x, out=out)  # type: ignore[return-value]
 
 
-def acos(x: VariableLikeType,
-         *,
-         out: Optional[VariableLikeType] = None) -> VariableLikeType:
+def acos(
+    x: VariableLikeType, *, out: VariableLikeType | None = None
+) -> VariableLikeType:
     """Element-wise inverse cosine.
 
     Parameters
@@ -114,12 +113,12 @@ def acos(x: VariableLikeType,
     : Same type as input
         The inverse cosine values of the input in radians.
     """
-    return _call_cpp_func(_cpp.acos, x, out=out)
+    return _call_cpp_func(_cpp.acos, x, out=out)  # type: ignore[return-value]
 
 
-def atan(x: VariableLikeType,
-         *,
-         out: Optional[VariableLikeType] = None) -> VariableLikeType:
+def atan(
+    x: VariableLikeType, *, out: VariableLikeType | None = None
+) -> VariableLikeType:
     """Element-wise inverse tangent.
 
     Parameters
@@ -134,13 +133,10 @@ def atan(x: VariableLikeType,
     : Same type as input
         The inverse tangent values of the input in radians.
     """
-    return _call_cpp_func(_cpp.atan, x, out=out)
+    return _call_cpp_func(_cpp.atan, x, out=out)  # type: ignore[return-value]
 
 
-def atan2(*,
-          y: _cpp.Variable,
-          x: _cpp.Variable,
-          out: Optional[_cpp.Variable] = None) -> _cpp.Variable:
+def atan2(*, y: Variable, x: Variable, out: Variable | None = None) -> Variable:
     """Element-wise inverse tangent of y/x determining the correct quadrant.
 
     Parameters
@@ -165,4 +161,4 @@ def atan2(*,
     numpy.arctan2:
         The equivalent in NumPy with additional explanations.
     """
-    return _call_cpp_func(_cpp.atan2, y=y, x=x, out=out)
+    return _call_cpp_func(_cpp.atan2, y=y, x=x, out=out)  # type: ignore[return-value]
