@@ -1,31 +1,34 @@
 # SPDX-License-Identifier: BSD-3-Clause
 # Copyright (c) 2023 Scipp contributors (https://github.com/scipp)
 # @author Matthew Andrew
-# flake8: noqa: E501
+# ruff: noqa: E501
 
 from __future__ import annotations
 
 import warnings
+from collections.abc import Iterable, Sequence
 from contextlib import contextmanager
-from typing import Any, Iterable, Optional, Sequence, TypeVar, Union
+from typing import Any, Generator, SupportsIndex, TypeVar
 
 import numpy as _np
 from numpy.typing import ArrayLike
 
 from .._scipp import core as _cpp
 from ..typing import DTypeLike
-from ..units import default_unit
+from ..units import DefaultUnit, default_unit
 from ._sizes import _parse_dims_shape_sizes
 from .cpp_classes import DType, Unit, Variable
 
-NumberOrVar = TypeVar('NumberOrVar', Union[int, float], Variable)
+NumberOrVar = TypeVar('NumberOrVar', int, float, Variable)
 
 
-def scalar(value: Any,
-           *,
-           variance: Any = None,
-           unit: Union[Unit, str, None] = default_unit,
-           dtype: Optional[DTypeLike] = None) -> Variable:
+def scalar(
+    value: Any,
+    *,
+    variance: Any = None,
+    unit: Unit | str | DefaultUnit | None = default_unit,
+    dtype: DTypeLike | None = None,
+) -> Variable:
     """Constructs a zero dimensional :class:`Variable` with a unit and optional
     variance.
 
@@ -57,30 +60,28 @@ def scalar(value: Any,
     With deduced dtype and default unit:
 
       >>> sc.scalar(3.14)
-      <scipp.Variable> ()    float64  [dimensionless]  [3.14]
+      <scipp.Variable> ()    float64  [dimensionless]  3.14
 
       >>> sc.scalar('a string')
-      <scipp.Variable> ()     string           <no unit>  ["a string"]
+      <scipp.Variable> ()     string           <no unit>  "a string"
 
     Or specifying a unit and dtype:
 
       >>> sc.scalar(3.14, unit='m', dtype=int)
-      <scipp.Variable> ()      int64              [m]  [3]
+      <scipp.Variable> ()      int64              [m]  3
 
     Calling ``scalar`` with a list (or similar array-like object) will store that
     object in a scalar variable and *not* create an array variable:
 
       >>> sc.scalar([1, 2, 3])
-      <scipp.Variable> ()   PyObject           <no unit>  [[1, 2, 3]]
+      <scipp.Variable> ()   PyObject           <no unit>  [1, 2, 3]
     """
-    return _cpp.Variable(dims=(),
-                         values=value,
-                         variances=variance,
-                         unit=unit,
-                         dtype=dtype)
+    return _cpp.Variable(  # type: ignore[no-any-return]
+        dims=(), values=value, variances=variance, unit=unit, dtype=dtype
+    )
 
 
-def index(value: Any, *, dtype: Optional[DTypeLike] = None) -> Variable:
+def index(value: Any, *, dtype: DTypeLike | None = None) -> Variable:
     """Constructs a zero dimensional :class:`Variable` representing an index.
 
     This is equivalent to calling :py:func:`scipp.scalar` with unit=None.
@@ -106,18 +107,20 @@ def index(value: Any, *, dtype: Optional[DTypeLike] = None) -> Variable:
     --------
 
       >>> sc.index(123)
-      <scipp.Variable> ()      int64           <no unit>  [123]
+      <scipp.Variable> ()      int64           <no unit>  123
     """
     return scalar(value=value, dtype=dtype, unit=None)
 
 
-def zeros(*,
-          dims: Sequence[str] = None,
-          shape: Sequence[int] = None,
-          sizes: dict = None,
-          unit: Union[Unit, str, None] = default_unit,
-          dtype: DTypeLike = DType.float64,
-          with_variances: bool = False) -> Variable:
+def zeros(
+    *,
+    dims: Sequence[str] | None = None,
+    shape: Sequence[int] | None = None,
+    sizes: dict[str, int] | None = None,
+    unit: Unit | str | DefaultUnit | None = default_unit,
+    dtype: DTypeLike = DType.float64,
+    with_variances: bool = False,
+) -> Variable:
     """Constructs a :class:`Variable` with default initialized values with
     given dimension labels and shape.
 
@@ -165,19 +168,23 @@ def zeros(*,
       <scipp.Variable> (z: 3)      int64             [kg]  [0, 0, 0]
     """
 
-    return _cpp.zeros(**_parse_dims_shape_sizes(dims, shape, sizes),
-                      unit=unit,
-                      dtype=dtype,
-                      with_variances=with_variances)
+    return _cpp.zeros(  # type: ignore[no-any-return]
+        **_parse_dims_shape_sizes(dims, shape, sizes),
+        unit=unit,
+        dtype=dtype,
+        with_variances=with_variances,
+    )
 
 
-def ones(*,
-         dims: Sequence[str] = None,
-         shape: Sequence[int] = None,
-         sizes: dict = None,
-         unit: Union[Unit, str, None] = default_unit,
-         dtype: DTypeLike = DType.float64,
-         with_variances: bool = False) -> Variable:
+def ones(
+    *,
+    dims: Sequence[str] | None = None,
+    shape: Sequence[int] | None = None,
+    sizes: dict[str, int] | None = None,
+    unit: Unit | str | DefaultUnit | None = default_unit,
+    dtype: DTypeLike = DType.float64,
+    with_variances: bool = False,
+) -> Variable:
     """Constructs a :class:`Variable` with values initialized to 1 with
     given dimension labels and shape.
 
@@ -222,19 +229,24 @@ def ones(*,
       >>> sc.ones(sizes={'z': 3}, unit='kg', dtype=int)
       <scipp.Variable> (z: 3)      int64             [kg]  [1, 1, 1]
     """
-    return _cpp.ones(**_parse_dims_shape_sizes(dims, shape, sizes),
-                     unit=unit,
-                     dtype=dtype,
-                     with_variances=with_variances)
+    return _cpp.ones(  # type: ignore[no-any-return]
+        **_parse_dims_shape_sizes(dims, shape, sizes),
+        unit=unit,
+        dtype=dtype,
+        with_variances=with_variances,
+    )
 
 
-def empty(*,
-          dims: Sequence[str] = None,
-          shape: Sequence[int] = None,
-          sizes: dict = None,
-          unit: Union[Unit, str, None] = default_unit,
-          dtype: DTypeLike = DType.float64,
-          with_variances: bool = False) -> Variable:
+def empty(
+    *,
+    dims: Sequence[str] | None = None,
+    shape: Sequence[int] | None = None,
+    sizes: dict[str, int] | None = None,
+    unit: Unit | str | DefaultUnit | None = default_unit,
+    dtype: DTypeLike = DType.float64,
+    with_variances: bool = False,
+    aligned: bool = True,
+) -> Variable:
     """Constructs a :class:`Variable` with uninitialized values with given
     dimension labels and shape.
 
@@ -261,6 +273,8 @@ def empty(*,
         Type of underlying data.
     with_variances:
         If True includes uninitialized variances.
+    aligned:
+        Initial value for the alignment flag.
 
     Returns
     -------
@@ -279,20 +293,25 @@ def empty(*,
       >>> var
       <scipp.Variable> (x: 4)    float64  [dimensionless]  [2, 2, 2, 2]
     """
-    return _cpp.empty(**_parse_dims_shape_sizes(dims, shape, sizes),
-                      unit=unit,
-                      dtype=dtype,
-                      with_variances=with_variances)
+    return _cpp.empty(  # type: ignore[no-any-return]
+        **_parse_dims_shape_sizes(dims, shape, sizes),
+        unit=unit,
+        dtype=dtype,
+        with_variances=with_variances,
+        aligned=aligned,
+    )
 
 
-def full(*,
-         value: Any,
-         variance: Any = None,
-         dims: Sequence[str] = None,
-         shape: Sequence[int] = None,
-         sizes: dict = None,
-         unit: Union[Unit, str, None] = default_unit,
-         dtype: Optional[DTypeLike] = None) -> Variable:
+def full(
+    *,
+    value: Any,
+    variance: Any = None,
+    dims: Sequence[str] | None = None,
+    shape: Sequence[int] | None = None,
+    sizes: dict[str, int] | None = None,
+    unit: Unit | str | DefaultUnit | None = default_unit,
+    dtype: DTypeLike | None = None,
+) -> Variable:
     """Constructs a :class:`Variable` with values initialized to the specified
     value with given dimension labels and shape.
 
@@ -339,56 +358,16 @@ def full(*,
       >>> sc.full(value=2.0, variance=0.1, sizes={'x': 3}, unit='s')
       <scipp.Variable> (x: 3)    float64              [s]  [2, 2, 2]  [0.1, 0.1, 0.1]
     """
-    return scalar(value=value, variance=variance, unit=unit, dtype=dtype)\
-        .broadcast(**_parse_dims_shape_sizes(dims, shape, sizes)).copy()
+    return (
+        scalar(value=value, variance=variance, unit=unit, dtype=dtype)
+        .broadcast(**_parse_dims_shape_sizes(dims, shape, sizes))
+        .copy()
+    )
 
 
-def matrix(*,
-           unit: Union[Unit, str, None] = default_unit,
-           value: Union[_np.ndarray, list]) -> Variable:
-    """Constructs a zero dimensional :class:`Variable` holding a single 3x3
-    matrix.
-
-
-    :param value: Initial value, a list or 1-D NumPy array.
-    :param unit: Optional, unit. Default=dimensionless
-    :returns: A scalar (zero-dimensional) Variable.
-
-    :seealso: :py:func:`scipp.matrices`
-    """
-    warnings.warn(
-        "sc.matrix() has been deprecated in favour of "
-        "sc.spatial.linear_transform(), and will be removed in a future "
-        "version of scipp.", DeprecationWarning)
-    from ..spatial import linear_transform
-    return linear_transform(unit=unit, value=value)
-
-
-def matrices(*,
-             dims: Sequence[str],
-             unit: Union[Unit, str, None] = default_unit,
-             values: Union[_np.ndarray, list]) -> Variable:
-    """Constructs a :class:`Variable` with given dimensions holding an array
-    of 3x3 matrices.
-
-
-    :param dims: Dimension labels.
-    :param values: Initial values.
-    :param unit: Optional, data unit. Default=dimensionless
-
-    :seealso: :py:func:`scipp.matrix`
-    """
-    warnings.warn(
-        "sc.matrices() has been deprecated in favour of "
-        "sc.spatial.linear_transforms(), and will be removed in a future "
-        "version of scipp.", DeprecationWarning)
-    from ..spatial import linear_transforms
-    return linear_transforms(dims=dims, unit=unit, values=values)
-
-
-def vector(value: Union[_np.ndarray, list],
-           *,
-           unit: Union[Unit, str, None] = default_unit) -> Variable:
+def vector(
+    value: ArrayLike, *, unit: Unit | str | DefaultUnit | None = default_unit
+) -> Variable:
     """Constructs a zero dimensional :class:`Variable` holding a single length-3
     vector.
 
@@ -406,24 +385,29 @@ def vector(value: Union[_np.ndarray, list],
 
     See Also
     --------
-    scipp.vectors
+    scipp.vectors:
+        Construct an array of vectors.
+    scipp.spatial.as_vectors:
+        Construct vectors from Scipp Variables
 
     Examples
     --------
 
       >>> sc.vector(value=[1, 2, 3])
-      <scipp.Variable> ()    vector3  [dimensionless]  [(1, 2, 3)]
+      <scipp.Variable> ()    vector3  [dimensionless]  (1, 2, 3)
 
       >>> sc.vector(value=[4, 5, 6], unit='m')
-      <scipp.Variable> ()    vector3              [m]  [(4, 5, 6)]
+      <scipp.Variable> ()    vector3              [m]  (4, 5, 6)
     """
-    return _cpp.vectors(dims=[], unit=unit, values=value)
+    return vectors(dims=(), values=value, unit=unit)
 
 
-def vectors(*,
-            dims: Sequence[str],
-            values: Union[_np.ndarray, list],
-            unit: Union[Unit, str, None] = default_unit) -> Variable:
+def vectors(
+    *,
+    dims: Sequence[str],
+    values: ArrayLike,
+    unit: Unit | str | DefaultUnit | None = default_unit,
+) -> Variable:
     """Constructs a :class:`Variable` with given dimensions holding an array
     of length-3 vectors.
 
@@ -443,7 +427,10 @@ def vectors(*,
 
     See Also
     --------
-    scipp.vector
+    scipp.vector:
+        Construct a single vector.
+    scipp.spatial.as_vectors:
+        Construct vectors from Scipp Variables
 
     Examples
     --------
@@ -461,15 +448,19 @@ def vectors(*,
       >>> sc.vectors(dims=['x'], values=[[1, 2, 3], [4, 5, 6]], unit='mm')
       <scipp.Variable> (x: 2)    vector3             [mm]  [(1, 2, 3), (4, 5, 6)]
     """
-    return _cpp.vectors(dims=dims, unit=unit, values=values)
+    return _cpp.Variable(  # type: ignore[no-any-return]
+        dims=dims, values=values, unit=unit, dtype=DType.vector3
+    )
 
 
-def array(*,
-          dims: Iterable[str],
-          values: ArrayLike,
-          variances: Optional[ArrayLike] = None,
-          unit: Union[Unit, str, None] = default_unit,
-          dtype: Optional[DTypeLike] = None) -> Variable:
+def array(
+    *,
+    dims: Iterable[str],
+    values: ArrayLike,
+    variances: ArrayLike | None = None,
+    unit: Unit | str | DefaultUnit | None = default_unit,
+    dtype: DTypeLike | None = None,
+) -> Variable:
     """Constructs a :class:`Variable` with given dimensions, containing given
     values and optional variances.
 
@@ -529,31 +520,34 @@ def array(*,
       >>> sc.array(dims=['x'], values=[1.0, 2.0, 3.0], variances=[0.1, 0.2, 0.3])
       <scipp.Variable> (x: 3)    float64  [dimensionless]  [1, 2, 3]  [0.1, 0.2, 0.3]
     """
-    return _cpp.Variable(dims=dims,
-                         values=values,
-                         variances=variances,
-                         unit=unit,
-                         dtype=dtype)
+    return _cpp.Variable(  # type: ignore[no-any-return]
+        dims=dims, values=values, variances=variances, unit=unit, dtype=dtype
+    )
 
 
-def _expect_no_variances(args):
+def _expect_no_variances(args: dict[str, Variable | None]) -> None:
     has_variances = [
-        key for key, val in args.items()
+        key
+        for key, val in args.items()
         if val is not None and val.variances is not None
     ]
     if has_variances:
-        raise _cpp.VariancesError('Arguments cannot have variances. '
-                                  f'Passed variances in {has_variances}')
+        raise _cpp.VariancesError(
+            f'Arguments cannot have variances. Passed variances in {has_variances}'
+        )
 
 
 # Assumes that all arguments are Variable or None.
-def _ensure_same_unit(*, unit, args: dict):
+def _ensure_same_unit(
+    *, unit: Unit | str | DefaultUnit | None, args: dict[str, Variable | None]
+) -> tuple[dict[str, Variable | None], Unit | str | DefaultUnit | None]:
     if unit == default_unit:
         units = {key: val.unit for key, val in args.items() if val is not None}
         if len(set(units.values())) != 1:
             raise _cpp.UnitError(
                 f'All units of the following arguments must be equal: {units}. '
-                'You can specify a unit explicitly with the `unit` argument.')
+                'You can specify a unit explicitly with the `unit` argument.'
+            )
         unit = next(iter(units.values()))
     return {
         key: _cpp.to_unit(val, unit, copy=False).value if val is not None else None
@@ -562,29 +556,36 @@ def _ensure_same_unit(*, unit, args: dict):
 
 
 # Process arguments of arange, linspace, etc and return them as plain numbers or None.
-def _normalize_range_args(*, unit, **kwargs):
+def _normalize_range_args(
+    *, unit: Unit | str | DefaultUnit | None, **kwargs: Any
+) -> tuple[dict[str, Any], Unit | str | DefaultUnit | None]:
     is_var = {
         key: isinstance(val, _cpp.Variable)
-        for key, val in kwargs.items() if val is not None
+        for key, val in kwargs.items()
+        if val is not None
     }
     if any(is_var.values()):
         if not all(is_var.values()):
             arg_types = {key: type(val) for key, val in kwargs.items()}
-            raise TypeError('Either all of the following arguments or none have to '
-                            f'be variables: {arg_types}')
+            raise TypeError(
+                'Either all of the following arguments or none have to '
+                f'be variables: {arg_types}'
+            )
         _expect_no_variances(kwargs)
         return _ensure_same_unit(unit=unit, args=kwargs)
     return kwargs, unit
 
 
-def linspace(dim: str,
-             start: NumberOrVar,
-             stop: NumberOrVar,
-             num: int,
-             *,
-             endpoint: bool = True,
-             unit: Union[Unit, str, None] = default_unit,
-             dtype: Optional[DTypeLike] = None) -> Variable:
+def linspace(
+    dim: str,
+    start: NumberOrVar,
+    stop: NumberOrVar,
+    num: SupportsIndex,
+    *,
+    endpoint: bool = True,
+    unit: Unit | str | DefaultUnit | None = default_unit,
+    dtype: DTypeLike | None = None,
+) -> Variable:
     """Constructs a :class:`Variable` with `num` evenly spaced samples,
     calculated over the interval `[start, stop]`.
 
@@ -627,20 +628,24 @@ def linspace(dim: str,
       <scipp.Variable> (x: 4)    float64              [m]  [1.5, 2, 2.5, 3]
     """
     range_args, unit = _normalize_range_args(unit=unit, start=start, stop=stop)
-    return array(dims=[dim],
-                 values=_np.linspace(**range_args, num=num, endpoint=endpoint),
-                 unit=unit,
-                 dtype=dtype)
+    return array(
+        dims=[dim],
+        values=_np.linspace(**range_args, num=num, endpoint=endpoint),
+        unit=unit,
+        dtype=dtype,
+    )
 
 
-def geomspace(dim: str,
-              start: NumberOrVar,
-              stop: NumberOrVar,
-              num: int,
-              *,
-              endpoint: bool = True,
-              unit: Union[Unit, str, None] = default_unit,
-              dtype: Optional[DTypeLike] = None) -> Variable:
+def geomspace(
+    dim: str,
+    start: NumberOrVar,
+    stop: NumberOrVar,
+    num: int,
+    *,
+    endpoint: bool = True,
+    unit: Unit | str | DefaultUnit | None = default_unit,
+    dtype: DTypeLike | None = None,
+) -> Variable:
     """Constructs a :class:`Variable` with values spaced evenly on a log scale
     (a geometric progression).
 
@@ -687,21 +692,25 @@ def geomspace(dim: str,
       <scipp.Variable> (x: 3)    float64              [s]  [1, 10, 100]
     """
     range_args, unit = _normalize_range_args(unit=unit, start=start, stop=stop)
-    return array(dims=[dim],
-                 values=_np.geomspace(**range_args, num=num, endpoint=endpoint),
-                 unit=unit,
-                 dtype=dtype)
+    return array(
+        dims=[dim],
+        values=_np.geomspace(**range_args, num=num, endpoint=endpoint),
+        unit=unit,
+        dtype=dtype,
+    )
 
 
-def logspace(dim: str,
-             start: NumberOrVar,
-             stop: NumberOrVar,
-             num: int,
-             *,
-             endpoint: bool = True,
-             base: Union[int, float] = 10.0,
-             unit: Union[Unit, str, None] = default_unit,
-             dtype: Optional[DTypeLike] = None) -> Variable:
+def logspace(
+    dim: str,
+    start: NumberOrVar,
+    stop: NumberOrVar,
+    num: int,
+    *,
+    endpoint: bool = True,
+    base: float = 10.0,
+    unit: Unit | str | DefaultUnit | None = default_unit,
+    dtype: DTypeLike | None = None,
+) -> Variable:
     """Constructs a :class:`Variable` with values spaced evenly on a log scale.
 
     This is similar to :py:func:`scipp.geomspace`, but with endpoints specified
@@ -756,22 +765,23 @@ def logspace(dim: str,
     """
     # Passing unit='one' enforces that start and stop are dimensionless.
     range_args, _ = _normalize_range_args(unit='one', start=start, stop=stop)
-    return array(dims=[dim],
-                 values=_np.logspace(**range_args,
-                                     num=num,
-                                     base=base,
-                                     endpoint=endpoint),
-                 unit=unit,
-                 dtype=dtype)
+    return array(
+        dims=[dim],
+        values=_np.logspace(**range_args, num=num, base=base, endpoint=endpoint),
+        unit=unit,
+        dtype=dtype,
+    )
 
 
-def arange(dim: str,
-           start: Union[NumberOrVar, _np.datetime64, str],
-           stop: Optional[Union[NumberOrVar, _np.datetime64, str]] = None,
-           step: Optional[NumberOrVar] = None,
-           *,
-           unit: Union[Unit, str, None] = default_unit,
-           dtype: Optional[DTypeLike] = None) -> Variable:
+def arange(
+    dim: str,
+    start: NumberOrVar | _np.datetime64 | str,
+    stop: NumberOrVar | _np.datetime64 | str | None = None,
+    step: NumberOrVar | None = None,
+    *,
+    unit: Unit | str | DefaultUnit | None = default_unit,
+    dtype: DTypeLike | None = None,
+) -> Variable:
     """Creates a :class:`Variable` with evenly spaced values within a given interval.
 
     Values are generated within the half-open interval [start, stop).
@@ -839,47 +849,55 @@ def arange(dim: str,
     define the unit. The ``step`` must have the same unit.
     """
     if dtype == 'datetime64' and isinstance(start, str):
-        start = datetime(start)
-        stop = stop if stop is None else datetime(stop)
-    range_args, unit = _normalize_range_args(unit=unit,
-                                             start=start,
-                                             stop=stop,
-                                             step=step)
-    args = [x for x in [start, stop, step] if x is not None]
+        start = datetime(start)  # type: ignore[assignment]
+        if not isinstance(stop, str) and stop is not None:
+            raise TypeError(
+                'Argument `stop` must be a string or `None` if `start` is a string.'
+            )
+        stop = stop if stop is None else datetime(stop)  # type: ignore[assignment]
+    range_args, unit = _normalize_range_args(
+        unit=unit, start=start, stop=stop, step=step
+    )
     types = [
         x.values.dtype if isinstance(x, _cpp.Variable) else _np.asarray(x).dtype
-        for x in args
+        for x in (start, stop, step)
+        if x is not None
     ]
     if dtype is None:
         candidates = set(types)
         if len(candidates) == 1:
             dtype = next(iter(candidates))
-    if dtype is not None and dtype != str('datetime64'):
+    if dtype is not None and dtype != 'datetime64':
         numpy_dtype = str(dtype) if isinstance(dtype, DType) else dtype
     else:
         numpy_dtype = None
-    return array(dims=[dim],
-                 values=_np.arange(**range_args, dtype=numpy_dtype),
-                 unit=unit,
-                 dtype=dtype)
+    return array(
+        dims=[dim],
+        values=_np.arange(**range_args, dtype=numpy_dtype),
+        unit=unit,
+        dtype=dtype,
+    )
 
 
 @contextmanager
-def _timezone_warning_as_error():
+def _timezone_warning_as_error() -> Generator[None, None, None]:
     with warnings.catch_warnings():
-        warnings.filterwarnings('error',
-                                category=DeprecationWarning,
-                                message='parsing timezone')
+        warnings.filterwarnings(
+            'error', category=DeprecationWarning, message='parsing timezone'
+        )
         try:
             yield
         except DeprecationWarning:
             raise ValueError(
-                'Parsing timezone aware datetimes is not supported') from None
+                'Parsing timezone aware datetimes is not supported'
+            ) from None
 
 
-def datetime(value: Union[str, int, _np.datetime64],
-             *,
-             unit: Optional[Union[Unit, str, None]] = default_unit) -> Variable:
+def datetime(
+    value: str | int | _np.datetime64,
+    *,
+    unit: Unit | str | DefaultUnit | None = default_unit,
+) -> Variable:
     """Constructs a zero dimensional :class:`Variable` with a dtype of datetime64.
 
     Parameters
@@ -911,11 +929,11 @@ def datetime(value: Union[str, int, _np.datetime64],
     --------
 
       >>> sc.datetime('2021-01-10T14:16:15')
-      <scipp.Variable> ()  datetime64              [s]  [2021-01-10T14:16:15]
+      <scipp.Variable> ()  datetime64              [s]  2021-01-10T14:16:15
       >>> sc.datetime('2021-01-10T14:16:15', unit='ns')
-      <scipp.Variable> ()  datetime64             [ns]  [2021-01-10T14:16:15.000000000]
+      <scipp.Variable> ()  datetime64             [ns]  2021-01-10T14:16:15.000000000
       >>> sc.datetime(1610288175, unit='s')
-      <scipp.Variable> ()  datetime64              [s]  [2021-01-10T14:16:15]
+      <scipp.Variable> ()  datetime64              [s]  2021-01-10T14:16:15
 
     Get the current time:
 
@@ -927,10 +945,12 @@ def datetime(value: Union[str, int, _np.datetime64],
     return scalar(value, unit=unit, dtype=_cpp.DType.datetime64)
 
 
-def datetimes(*,
-              dims,
-              values: ArrayLike,
-              unit: Optional[Union[Unit, str, None]] = default_unit) -> Variable:
+def datetimes(
+    *,
+    dims: Sequence[str],
+    values: ArrayLike,
+    unit: Unit | str | DefaultUnit | None = default_unit,
+) -> Variable:
     """Constructs an array :class:`Variable` with a dtype of datetime64.
 
     Parameters
@@ -970,11 +990,12 @@ def datetimes(*,
     """
     np_unit_str = f'[{u}]' if (u := _cpp.to_numpy_time_string(unit)) else ''
     with _timezone_warning_as_error():
-        return array(dims=dims,
-                     values=_np.asarray(values, dtype=f'datetime64{np_unit_str}'))
+        return array(
+            dims=dims, values=_np.asarray(values, dtype=f'datetime64{np_unit_str}')
+        )
 
 
-def epoch(*, unit: Union[Unit, str]) -> Variable:
+def epoch(*, unit: Unit | str | DefaultUnit | None) -> Variable:
     """Constructs a zero dimensional :class:`Variable` with a dtype of
     datetime64 that contains Scipp's epoch.
 
@@ -1001,6 +1022,6 @@ def epoch(*, unit: Union[Unit, str]) -> Variable:
     --------
 
       >>> sc.epoch(unit='s')
-      <scipp.Variable> ()  datetime64              [s]  [1970-01-01T00:00:00]
+      <scipp.Variable> ()  datetime64              [s]  1970-01-01T00:00:00
     """
     return scalar(0, unit=unit, dtype=_cpp.DType.datetime64)
