@@ -10,14 +10,16 @@ See the https://scipp.github.io/reference/logging.html for an overview.
 import html
 import logging
 import time
+from collections.abc import Iterable
 from copy import copy
 from dataclasses import dataclass
-from typing import Any, Dict, Optional, Tuple
+from typing import Any
 
 from .core import DataArray, Dataset, Variable
-from .html import make_html
-from .html.resources import load_style
+from .typing import VariableLike
 from .utils import running_in_jupyter
+from .visualization import make_html
+from .visualization.resources import load_style
 
 
 def get_logger() -> logging.Logger:
@@ -32,27 +34,30 @@ class WidgetLogRecord:
     """
     Preprocessed data for display in ``LogWidget``.
     """
+
     name: str
     levelname: str
     time_stamp: str
     message: str
 
 
-def make_log_widget(**widget_kwargs):
+def make_log_widget(**widget_kwargs: object) -> object:
     if not running_in_jupyter():
-        raise RuntimeError('Cannot create a logging widget because '
-                           'Python is not running in a Jupyter notebook.')
+        raise RuntimeError(
+            'Cannot create a logging widget because '
+            'Python is not running in a Jupyter notebook.'
+        )
 
     from ipywidgets import HTML
 
-    class LogWidget(HTML):
+    class LogWidget(HTML):  # type: ignore[misc]
         """
         Widget that displays log messages in a table.
 
         :seealso: :py:class:`scipp.logging.WidgetHandler`
         """
 
-        def __init__(self, **kwargs):
+        def __init__(self, **kwargs: object) -> None:
             super().__init__(**kwargs)
             self._rows_str = ''
             self._update()
@@ -75,12 +80,15 @@ def make_log_widget(**widget_kwargs):
                 f'<td class="sc-log-level">{html.escape(record.levelname)}</td>'
                 f'<td class="sc-log-message">{record.message}</td>'
                 f'<td class="sc-log-name">&lt;{html.escape(record.name)}&gt;</td>'
-                '</tr>')
+                '</tr>'
+            )
 
         def _update(self) -> None:
-            self.value = ('<div class="sc-log">'
-                          f'<table class="sc-log">{self._rows_str}</table>'
-                          '</div>')
+            self.value = (
+                '<div class="sc-log">'
+                f'<table class="sc-log">{self._rows_str}</table>'
+                '</div>'
+            )
 
         def clear(self) -> None:
             """
@@ -92,7 +100,7 @@ def make_log_widget(**widget_kwargs):
     return LogWidget(**widget_kwargs)
 
 
-def get_log_widget():
+def get_log_widget() -> object:
     """
     Return the log widget used by the Scipp logger.
     If multiple widget handlers are installed, only the first one is returned.
@@ -112,11 +120,13 @@ def display_logs() -> None:
     widget = get_log_widget()
     if widget is None:
         raise RuntimeError(
-            'Cannot display log widgets because no widget handler is installed.')
+            'Cannot display log widgets because no widget handler is installed.'
+        )
 
     from IPython.display import display
     from ipywidgets import HTML, VBox
-    display(VBox([HTML(value=load_style()), widget]).add_class('sc-log-wrap'))
+
+    display(VBox([HTML(value=load_style()), widget]).add_class('sc-log-wrap'))  # type: ignore[no-untyped-call]
 
 
 def clear_log_widget() -> None:
@@ -127,14 +137,14 @@ def clear_log_widget() -> None:
     widget = get_log_widget()
     if widget is None:
         return
-    widget.clear()
+    widget.clear()  # type: ignore[attr-defined]
 
 
-def _has_html_repr(x: Any) -> bool:
-    return isinstance(x, (DataArray, Dataset, Variable))
+def _has_html_repr(x: object) -> bool:
+    return isinstance(x, DataArray | Dataset | Variable)
 
 
-def _make_html(x) -> str:
+def _make_html(x: VariableLike) -> str:
     return f'<div class="sc-log-html-payload">{make_html(x)}</div>'
 
 
@@ -144,19 +154,21 @@ def _make_html(x) -> str:
 class _ReplacementPattern:
     _PATTERN = '$__SCIPP_HTML_REPLACEMENT_{:02d}__'
 
-    def __init__(self, i: int, arg: Any):
+    def __init__(self, i: int, arg: Any) -> None:
         self._i = i
         self._arg = arg
 
-    def __str__(self):
+    def __str__(self) -> str:
         return self._PATTERN.format(self._i)
 
-    def __repr__(self):
+    def __repr__(self) -> str:
         return repr(self._arg)
 
 
-def _preprocess_format_args(args) -> Tuple[Tuple, Dict[str, Any]]:
-    format_args = []
+def _preprocess_format_args(
+    args: Iterable[object],
+) -> tuple[tuple[object, ...], dict[str, object]]:
+    format_args: list[object] = []
     replacements = {}
     for i, arg in enumerate(args):
         if _has_html_repr(arg):
@@ -168,13 +180,16 @@ def _preprocess_format_args(args) -> Tuple[Tuple, Dict[str, Any]]:
     return tuple(format_args), replacements
 
 
-def _replace_html_repr(message: str, replacements: Dict[str, str]) -> str:
+def _replace_html_repr(message: str, replacements: dict[str, object]) -> str:
     # Do separate check `key in message` in order to avoid calling
     # _make_html unnecessarily. Linear string searches are likely less
     # expensive than HTML formatting.
     for key, repl in replacements.items():
         if key in message:
-            message = message.replace(key, _make_html(repl))
+            if isinstance(repl, Variable | DataArray | Dataset):
+                message = message.replace(key, _make_html(repl))
+            else:
+                message = message.replace(key, str(repl))
     return message
 
 
@@ -191,25 +206,34 @@ class WidgetHandler(logging.Handler):
     repr using ``str(x)`` and ``repr(x)`` is inaccessible.
     """
 
-    def __init__(self, level: int, widget):
+    def __init__(self, level: int, widget: object) -> None:
         super().__init__(level)
-        self.widget = widget
-        self._rows = []
+        self.widget: Any = widget  # The type is not nameable here.
 
-    def format(self, record: logging.LogRecord) -> WidgetLogRecord:
-        """
-        Format the specified record for consumption by a ``LogWidget``.
-        """
-        message = self._format_html(record) if _has_html_repr(
-            record.msg) else self._format_text(record)
-        return WidgetLogRecord(name=record.name,
-                               levelname=record.levelname,
-                               time_stamp=time.strftime('%Y-%m-%dT%H:%M:%S',
-                                                        time.localtime(record.created)),
-                               message=message)
+    def _format_record(self, record: logging.LogRecord) -> WidgetLogRecord:
+        """Format the specified record for consumption by a ``LogWidget``."""
+        message = (
+            self._format_html(record)
+            if _has_html_repr(record.msg)
+            else self._format_text(record)
+        )
+        return WidgetLogRecord(
+            name=record.name,
+            levelname=record.levelname,
+            time_stamp=time.strftime(
+                '%Y-%m-%dT%H:%M:%S', time.localtime(record.created)
+            ),
+            message=message,
+        )
 
     def _format_text(self, record: logging.LogRecord) -> str:
-        args, replacements = _preprocess_format_args(record.args)
+        if record.args is None:
+            record_args: Iterable[object] = ()
+        elif isinstance(record.args, tuple):
+            record_args = record.args
+        else:
+            record_args = record.args.values()
+        args, replacements = _preprocess_format_args(record_args)
         record = copy(record)
         record.args = tuple(args)
         message = html.escape(super().format(record))
@@ -219,13 +243,13 @@ class WidgetHandler(logging.Handler):
     def _format_html(record: logging.LogRecord) -> str:
         if record.args:
             raise TypeError('not all arguments converted during string formatting')
-        return _make_html(record.msg)
+        return _make_html(record.msg)  # type: ignore[arg-type]
 
     def emit(self, record: logging.LogRecord) -> None:
         """
         Send the formatted record to the widget.
         """
-        self.widget.add_message(self.format(record))
+        self.widget.add_message(self._format_record(record))
 
 
 def make_widget_handler() -> WidgetHandler:
@@ -237,15 +261,18 @@ def make_widget_handler() -> WidgetHandler:
     return handler
 
 
-def get_widget_handler() -> Optional[WidgetHandler]:
+def get_widget_handler() -> WidgetHandler | None:
     """
     Return the widget handler installed in the Scipp logger.
     If multiple widget handlers are installed, only the first one is returned.
     If no widget handler is installed, returns ``None``.
     """
     try:
-        return next(  # type: ignore
-            filter(lambda handler: isinstance(handler, WidgetHandler),
-                   get_logger().handlers))
+        return next(  # type: ignore[return-value]
+            filter(
+                lambda handler: isinstance(handler, WidgetHandler),
+                get_logger().handlers,
+            )
+        )
     except StopIteration:
         return None
