@@ -1,42 +1,40 @@
 # SPDX-License-Identifier: BSD-3-Clause
 # Copyright (c) 2023 Scipp contributors (https://github.com/scipp)
 # @author Simon Heybrock
-from typing import Dict, List, Tuple, Union
+
+from typing import Any
 
 from .core import DataArray, Dataset, Variable
 
 
-def serialize(var: Union[Variable, DataArray, Dataset]) -> Tuple[Dict, List[bytes]]:
+def serialize(obj: object) -> tuple[dict[str, Any], list[bytes]]:
     """Serialize Scipp object."""
     from io import BytesIO
 
-    import h5py
+    from .io.hdf5 import save_hdf5
 
-    from .io.hdf5 import HDF5IO
-    header = {}
+    header: dict[str, Any] = {}
     buf = BytesIO()
-    with h5py.File(buf, "w") as f:
-        HDF5IO.write(f, var)
+    save_hdf5(obj, buf)
     frames = [buf.getvalue()]
     return header, frames
 
 
-def deserialize(header: Dict,
-                frames: List[bytes]) -> Union[Variable, DataArray, Dataset]:
+def deserialize(header: dict[str, Any], frames: list[bytes]) -> object:
     """Deserialize Scipp object."""
     from io import BytesIO
 
-    import h5py
+    from .io.hdf5 import load_hdf5
 
-    from .io.hdf5 import HDF5IO
-    return HDF5IO.read(h5py.File(BytesIO(frames[0]), "r"))
+    return load_hdf5(BytesIO(frames[0]))
 
 
 try:
     from distributed.protocol import register_serialization
+
     register_serialization(Variable, serialize, deserialize)
     register_serialization(DataArray, serialize, deserialize)
     register_serialization(Dataset, serialize, deserialize)
 except ImportError:
     pass
-__all__ = ['serialize', 'deserialize']
+__all__ = ['deserialize', 'serialize']
