@@ -40,11 +40,11 @@ Disabling autoplot
 ``autoplot`` can be disabled by using the ``.. autoplot-disable::`` directive
 anywhere (on its own line) in a docstring.
 This will disable autoplot for the entire docstring.
-"""  # noqa: E501
+"""
 
 import re
-from typing import List, Optional
 
+from sphinx.application import Sphinx
 from sphinx.util.docutils import SphinxDirective
 
 # If this pattern matches any line of a code block,
@@ -56,7 +56,7 @@ PLOT_DIRECTIVE_PATTERN = re.compile(r'^\s*\.\. plot::.*')
 DISABLE_DIRECTIVE_PATTERN = re.compile(r'^\s*\.\. autoplot-disable::.*')
 
 
-def _previous_nonempty_line(lines: List[str], index: int) -> Optional[str]:
+def _previous_nonempty_line(lines: list[str], index: int) -> str | None:
     index -= 1
     while index >= 0 and not lines[index].strip():
         index -= 1
@@ -67,7 +67,7 @@ def _indentation_of(s: str) -> int:
     return len(s) - len(s.lstrip())
 
 
-def _process_block(lines: List[str], begin: int, end: int) -> List[str]:
+def _process_block(lines: list[str], begin: int, end: int) -> list[str]:
     block = lines[begin:end]
     if PLOT_PATTERN.search(block[-1]) is not None:
         prev = _previous_nonempty_line(lines, begin)
@@ -77,8 +77,9 @@ def _process_block(lines: List[str], begin: int, end: int) -> List[str]:
             # Adding some indentation here fixes that but also modifies indentation
             # if the block is already indented. That should not cause problems.
             return [
-                ' ' * _indentation_of(prev) + '.. plot::', '',
-                *map(lambda x: '    ' + x, block)
+                ' ' * _indentation_of(prev) + '.. plot::',
+                '',
+                *('    ' + x for x in block),
             ]
     return block
 
@@ -92,9 +93,16 @@ def _is_part_of_block(line: str) -> bool:
     return stripped.startswith('>>>') or stripped.startswith('...')
 
 
-def add_plot_directives(app, what, name, obj, options, lines: List[str]):
+def add_plot_directives(
+    app: object,
+    what: object,
+    name: object,
+    obj: object,
+    options: object,
+    lines: list[str],
+) -> None:
     new_lines = []
-    block_begin = None
+    block_begin: int | None = None
     for i, line in enumerate(lines):
         if DISABLE_DIRECTIVE_PATTERN.match(line):
             return
@@ -119,11 +127,11 @@ def add_plot_directives(app, what, name, obj, options, lines: List[str]):
 class AutoplotDisable(SphinxDirective):
     has_content = False
 
-    def run(self):
+    def run(self) -> list[object]:
         return []
 
 
-def setup(app):
+def setup(app: Sphinx) -> dict[str, int | bool]:
     app.add_directive('autoplot-disable', AutoplotDisable)
     app.connect('autodoc-process-docstring', add_plot_directives)
     return {'version': 1, 'parallel_read_safe': True}
