@@ -30,14 +30,29 @@ Special:
     :py:class:`scipp.Unit` to construct other units.
 """
 
+from collections.abc import Generator, Iterator
 from contextlib import contextmanager
-from typing import Dict, Iterable, Tuple, Union
 
 from .._scipp.core import add_unit_alias as _add_unit_alias
 from .._scipp.core import clear_unit_aliases as _clear_unit_aliases
-from .._scipp.core.units import K, angstrom, counts, default_unit, deg, dimensionless, \
-    kg, m, meV, mm, ns, one, rad, s, us  # NOQA
-from ..core.cpp_classes import Unit, Variable, VariancesError
+from .._scipp.core.units import (
+    K,
+    angstrom,
+    counts,
+    default_unit,
+    deg,
+    dimensionless,
+    kg,
+    m,
+    meV,
+    mm,
+    ns,
+    one,
+    rad,
+    s,
+    us,
+)
+from ..core.cpp_classes import DefaultUnit, Unit, UnitError, Variable, VariancesError
 
 
 class UnitAliases:
@@ -55,12 +70,12 @@ class UnitAliases:
     Instead, use it through :attr:`scipp.units.aliases`.
     """
 
-    def __init__(self):
-        if any(map(lambda x: isinstance(x, UnitAliases), globals().values())):
+    def __init__(self) -> None:
+        if any(isinstance(x, UnitAliases) for x in globals().values()):
             raise RuntimeError('There can be only one instance of _Aliases')
-        self._aliases: Dict[str, Unit] = {}
+        self._aliases: dict[str, Unit] = {}
 
-    def __setitem__(self, alias: str, unit: Union[str, Unit, Variable]):
+    def __setitem__(self, alias: str, unit: str | Unit | Variable) -> None:
         """Define a new unit alias."""
         unit = _build_unit(unit)
         if self._aliases.get(alias) == unit:
@@ -70,11 +85,11 @@ class UnitAliases:
         _add_unit_alias(name=alias, unit=unit)
         self._aliases[alias] = unit
 
-    def __delitem__(self, alias: str):
+    def __delitem__(self, alias: str) -> None:
         """Remove an existing alias."""
         self._del_aliases(alias)
 
-    def _del_aliases(self, *names: str):
+    def _del_aliases(self, *names: str) -> None:
         old_aliases = dict(self._aliases)
         for name in names:
             del old_aliases[name]
@@ -82,13 +97,13 @@ class UnitAliases:
         for name, unit in old_aliases.items():
             self[name] = unit
 
-    def clear(self):
+    def clear(self) -> None:
         """Remove all aliases."""
         self._aliases.clear()
         _clear_unit_aliases()
 
     @contextmanager
-    def scoped(self, **kwargs: Union[str, Unit]):
+    def scoped(self, **kwargs: str | Unit) -> Generator[None, None, None]:
         """Contextmanager to define temporary aliases.
 
         Defines new aliases based on ``kwargs`` for the duration of the context.
@@ -136,7 +151,7 @@ class UnitAliases:
           True
 
         Aliases defined within the context remain active
-        unless they clash with previous alises:
+        unless they clash with previous aliases:
 
           >>> sc.units.aliases.clear()
           >>> sc.units.aliases['speed'] = 'km/s'
@@ -149,8 +164,7 @@ class UnitAliases:
           True
         """
         overridden = {
-            name: unit
-            for name, unit in self._aliases.items() if name in kwargs
+            name: unit for name, unit in self._aliases.items() if name in kwargs
         }
         for name, unit in kwargs.items():
             self[name] = unit
@@ -159,38 +173,40 @@ class UnitAliases:
         for name, unit in overridden.items():
             self[name] = unit
 
-    def __iter__(self) -> Iterable[str]:
+    def __iter__(self) -> Iterator[str]:
         """Iterator over alias names."""
         yield from self.keys()
 
-    def keys(self) -> Iterable[str]:
+    def keys(self) -> Iterator[str]:
         """Iterator over alias names."""
         yield from self._aliases.keys()
 
-    def values(self) -> Iterable[Unit]:
+    def values(self) -> Iterator[Unit]:
         """Iterator over aliased units."""
         yield from self._aliases.values()
 
-    def items(self) -> Iterable[Tuple[str, Unit]]:
+    def items(self) -> Iterator[tuple[str, Unit]]:
         """Iterator over pairs of alias names and units."""
         yield from self._aliases.items()
 
     # Making copies would allow _Alias's internal map and
     # LLNL/Unit's global map to get out of sync.
-    def __copy__(self):
+    def __copy__(self) -> None:
         raise TypeError('UnitAliases is a singleton and must not be copied')
 
-    def __deepcopy__(self):
+    def __deepcopy__(self) -> None:
         raise TypeError('UnitAliases is a singleton and must not be copied')
 
 
-def _build_unit(x: Union[str, Unit, Variable]) -> Unit:
+def _build_unit(x: str | Unit | Variable) -> Unit:
     if isinstance(x, Unit):
         return x
     if isinstance(x, str):
         return Unit(x)
     if x.variance is not None:
         raise VariancesError('Cannot define a unit with a variance')
+    if x.unit is None:
+        raise UnitError('Cannot define a unit based on a variable without units')
     # Convert to float first to make sure the variable only contains a
     # multiplier and not a string that would be multiplied to the unit.
     return Unit(str(float(x.value))) * x.unit
@@ -200,6 +216,20 @@ aliases = UnitAliases()
 """Table of unit aliases."""
 
 __all__ = [
-    'angstrom', 'counts', 'default_unit', 'deg', 'dimensionless', 'kg', 'K', 'meV', 'm',
-    'one', 'rad', 's', 'us', 'ns', 'mm'
+    'DefaultUnit',
+    'K',
+    'angstrom',
+    'counts',
+    'default_unit',
+    'deg',
+    'dimensionless',
+    'kg',
+    'm',
+    'meV',
+    'mm',
+    'ns',
+    'one',
+    'rad',
+    's',
+    'us',
 ]
