@@ -3,13 +3,12 @@
 # @file
 # @author Neil Vaytet
 
-# flake8: noqa
+# ruff: noqa: F403
 """
 INTERNAL USE ONLY
 """
 
 from .collapse_and_slices import *
-from .colors import *
 from .comparison import *
 from .pyshell import *
 from .to_string import *
