@@ -1,5 +1,5 @@
 # SPDX-License-Identifier: BSD-3-Clause
-# Copyright (c) 2022 Scipp contributors (https://github.com/scipp)
+# Copyright (c) 2023 Scipp contributors (https://github.com/scipp)
 # @author Jan-Lukas Wynen
 
 
@@ -11,10 +11,13 @@ def running_in_jupyter() -> bool:
     but also Jupyter console and qtconsole.
     """
     try:
-        from IPython import get_ipython
         import ipykernel.zmqshell
+        from IPython import get_ipython  # type: ignore[attr-defined]
     except ImportError:
         # Cannot be Jupyter if IPython is not installed.
         return False
 
-    return isinstance(get_ipython(), ipykernel.zmqshell.ZMQInteractiveShell)
+    return isinstance(
+        get_ipython(),  # type: ignore[no-untyped-call]
+        ipykernel.zmqshell.ZMQInteractiveShell,
+    )
